<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use App\Models\Admin\CustomersModel;
use App\Models\Admin\DistrictsModel;

class Customers extends BaseController
{
    protected $auth;
    protected $config;
    public $customers;
    public $districts;

    public function __construct()
    {
        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->customers = new CustomersModel();
        $this->districts = new DistrictsModel();
        $this->db = \Config\Database::connect();
        $this->session = session();
    }

    // Customer list
    public function index()
    {
        $data['breadcrumb'] = 'Customers';
        $data['customers'] = $this->customers->getCustomers();
        $data['db'] = $this->db;
        return view('admin/customers/customer_list', $data);
    }

    // Customer Profile
    public function customer_profile($id)
    {
        $data['breadcrumb'] = 'Customer Profile';
        $data['customer'] = $this->customers->getCustomerProfile($id);
        $data['districts'] = $this->districts->getDistricts();
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'firstname' => 'required|min_length[3]',
                'lastname' => 'required|min_length[3]',
                'phone' => 'required|min_length[10]',
                'district' => 'required',
                'address' => 'required',
            ];

            if (!empty($this->request->getVar('email'))) {
                $rules = [
                    'email' => 'valid_email',
                ];
            }

            if (!empty($_FILES['image']['name'])) {
                $rules = [
                    'image' => 'uploaded[image]|max_size[image,2054]',
                ];
            }

            $messages = [
                'dob' => [
                    'required' => 'Your date of birth is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $findFile = $this->customers->getCustomerProfile($id);
                    $old_img = $findFile->profile_image;
                    $imagePath = 'public/uploads/users/profiles/' . $old_img;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/users/profiles/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'firstname' => $this->request->getVar('firstname'),
                    'lastname' => $this->request->getVar('lastname'),
                    'phone' => $this->request->getVar('phone'),
                    'address' => $this->request->getVar('address'),
                    'district' => $this->request->getVar('district'),
                    'dob' => $this->request->getVar('dob'),
                    'email' => $this->request->getVar('email'),
                    'billing_names' => $this->request->getVar('billing_names'),
                    'billing_phone' => $this->request->getVar('billing_phone'),
                    'billing_address' => $this->request->getVar('billing_address'),
                    'billing_district' => $this->request->getVar('billing_district'),
                    'billing_city' => $this->request->getVar('billing_city'),
                    'profile_image' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];

                $result = $this->customers->update($id, $data);

                if ($result) {

                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/users/profiles', $new_image);
                    }
                    $this->session->setTempdata('success', 'Settings updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Settings could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/customers/customer_profile', $data);
    }

    public function customer_password($id)
    {

        $data['breadcrumb'] = 'Customer Profile';
        $data['customer'] = $this->customers->getCustomerProfile($id);
        $data['districts'] = $this->districts->getDistricts();

        if ($this->request->getMethod() == 'post') {
            $config = config('Auth');
            if (
                (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                ||
                (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
            ) {
                $hashOptions = [
                    'memory_cost' => $config->hashMemoryCost,
                    'time_cost'   => $config->hashTimeCost,
                    'threads'     => $config->hashThreads,
                ];
            } else {
                $hashOptions = [
                    'cost' => $config->hashCost,
                ];
            }

            $rules = [
                'password' => 'required|min_length[8]',
                'confirm_password' => 'required|matches[password]',
            ];

            $messages = [
                'confirm_password' => [
                    'matches' => 'The two passwords donot match'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $password = $this->request->getVar('password');
                $data = [
                    'password_hash' => password_hash(self::preparePassword($password), $this->config->hashAlgorithm, $hashOptions),
                ];

                $result = $this->customers->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Password updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Password could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/customers/customer_profile', $data);
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }
}
