<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use App\Models\Admin\FarmerGroupsModel;

class FarmGroups extends BaseController
{
    private $groups;
    public $session;

    public function __construct()
    {
        $this->groups = new FarmerGroupsModel();
        $this->session = session();
    }

    // Farmer Groups
    public function index()
    {
        $data['breadcrumb'] = 'Farmer Groups';
        $data['groups'] = $this->groups->getGroups();
        return view('admin/groups/group_list', $data);
    }

    public function add_farm_group()
    {
        $data['breadcrumb'] = 'Add Group';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'is_unique[farm_groups.group_name]',
                'image' => 'uploaded[image]|max_size[image,120]|mime_in[image,image/png,image/jpg,image/gif]'
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Group Name is alreay used'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'group_name' => $this->request->getVar('name'),
                    'group_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'group_logo' => $new_image,
                ];

                $result = $this->groups->save($data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/farm-groups', $new_image);
                    }
                    $this->session->setTempdata('success', 'Farm Group added successfully', 3);
                    return redirect()->to(site_url('admin/add-farm-group'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Farm Group could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-farm-group'));
                }
            }
        }
        return view('admin/groups/add_farm_group', $data);
    }

    public function edit_farm_group($id)
    {

        $data['breadcrumb'] = 'Edit Group';
        $data['group'] = $this->groups->find($id);

        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Group Name is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $findFile = $this->groups->getGroupID($id);
                    $old_img = $findFile->group_logo;
                    $imagePath = 'public/uploads/farm-groups/' . $old_img;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/farm-groups/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'group_name' => $this->request->getVar('name'),
                    'group_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'group_logo' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];

                $result = $this->groups->update($id, $data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/farm-groups', $new_image);
                    }
                    $this->session->setTempdata('success', 'Farm Group updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Farm Group could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }

        return view('admin/groups/edit_farm_group', $data);
    }

    public function delete_farm_group($id)
    {
        // Unlink old image
        $findFile = $this->groups->getGroupID($id);
        $old_img = $findFile->group_logo;
        $imagePath = 'public/uploads/farm-groups/' . $old_img;

        if (file_exists($imagePath)) {
            unlink('public/uploads/farm-groups/' . $old_img);
        }

        $result = $this->groups->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Farmer Group Deleted successfully', 3);
            return redirect()->to(site_url('admin/farmer-groups'));
        }
    }
}
