<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use App\Models\Admin\FarmersModel;
use App\Models\Admin\DistrictsModel;
use App\Models\Admin\MarketsModel;

class Farmers extends BaseController
{
    protected $auth;
    protected $config;
    public $farmers;
    public $districts;
    public $markets;
    public $session;

    public function __construct()
    {
        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->farmers = new FarmersModel();
        $this->districts = new DistrictsModel();
        $this->markets = new MarketsModel();
        $this->db = \Config\Database::connect();
        $this->session = session();
    }

    // Farmers list
    public function index()
    {
        $data['breadcrumb'] = 'Farmers';
        $data['farmers'] = $this->farmers->getFarmers();
        return view('admin/providers/farmers', $data);
    }

    // Confirmed farmers
    public function confirmed_farmers()
    {
        $data['breadcrumb'] = 'Confirmed Farmers';
        $data['farmers'] = $this->farmers->getConfirmedFarmers();
        return view('admin/providers/confirmed_farmers', $data);
    }

    // Pending farmers
    public function pending_farmers()
    {
        $data['breadcrumb'] = 'Pending Farmers';
        $data['farmers'] = $this->farmers->getPendingFarmers();
        return view('admin/providers/pending_farmers', $data);
    }

    // Pending farmers
    public function declined_farmers()
    {
        $data['breadcrumb'] = 'Declined Farmers';
        $data['farmers'] = $this->farmers->getDeclinedFarmers();
        return view('admin/providers/declined_farmers', $data);
    }

    // Banned farmers
    public function banned_farmers()
    {
        $data['breadcrumb'] = 'Banned Farmers';
        $data['farmers'] = $this->farmers->getBannedFarmers();
        return view('admin/providers/banned_farmers', $data);
    }

    // Farmer Profile
    public function seller_profile($id)
    {
        $data['breadcrumb'] = 'Seller Profile';
        $data['seller'] = $this->farmers->getSellerProfile($id);
        $data['districts'] = $this->districts->getDistricts();
        $data['markets'] = $this->markets->getMarkets();
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'phone' => 'required|min_length[10]',
                'district' => 'required',
                'market' => 'required',
                'address' => 'required',
            ];

            if (!empty($this->request->getVar('email'))) {
                $rules = [
                    'email' => 'valid_email',
                ];
            }

            if (!empty($_FILES['image']['name'])) {
                $rules = [
                    'image' => 'uploaded[image]|max_size[image,2054]',
                ];
            }

            $messages = [
                'dob' => [
                    'required' => 'Your date of birth is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $findFile = $this->farmers->getSellerProfile($id);
                    $old_img = $findFile->profile_image;
                    $imagePath = 'public/uploads/users/profiles/' . $old_img;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/users/profiles/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'firstname' => $this->request->getVar('firstname'),
                    'lastname' => $this->request->getVar('lastname'),
                    'store_name' => $this->request->getVar('store_name'),
                    'slug' => url_title(strtolower($this->request->getVar('store_name')), '-', true),
                    'phone' => $this->request->getVar('phone'),
                    'address' => $this->request->getVar('address'),
                    'district' => $this->request->getVar('district'),
                    'store_description' => $this->request->getVar('store_description'),
                    'verification' => $this->request->getVar('verification'),
                    'market' => $this->request->getVar('market'),
                    'dob' => $this->request->getVar('dob'),
                    'email' => $this->request->getVar('email'),
                    'store_tagline' => $this->request->getVar('store_tagline'),
                    'profile_image' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];


                $result = $this->farmers->update($id, $data);

                if ($result) {

                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/users/profiles', $new_image);
                    }
                    $this->session->setTempdata('success', 'Settings updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Settings could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/providers/seller_profile', $data);
    }

    public function seller_password($id)
    {

        $data['breadcrumb'] = 'Seller Profile';
        $data['seller'] = $this->farmers->getSellerProfile($id);
        $data['districts'] = $this->districts->getDistricts();
        $data['markets'] = $this->markets->getMarkets();
        if ($this->request->getMethod() == 'post') {
            $config = config('Auth');
            if (
                (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                ||
                (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
            ) {
                $hashOptions = [
                    'memory_cost' => $config->hashMemoryCost,
                    'time_cost'   => $config->hashTimeCost,
                    'threads'     => $config->hashThreads,
                ];
            } else {
                $hashOptions = [
                    'cost' => $config->hashCost,
                ];
            }

            $rules = [
                'password' => 'required|min_length[8]',
                'confirm_password' => 'required|matches[password]',
            ];

            $messages = [
                'confirm_password' => [
                    'matches' => 'The two passwords donot match'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $password = $this->request->getVar('password');
                $data = [
                    'password_hash' => password_hash(self::preparePassword($password), $this->config->hashAlgorithm, $hashOptions),
                ];

                $result = $this->farmers->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Password updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Password could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/providers/seller_profile', $data);
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }

    public function download_doc($id)
    {
        helper('download');
        $fileinfo = $this->farmers->getSellerProfile($id);
        $doc = $fileinfo->document;
        // $docPath = readfile("./robots.txt");
        $docPath = 'public/uploads/users/sellers/docs/' . $doc;
        return $this->request->getFile($docPath);
        // return download($docPath, null);
        // force_download($docPath, NULL);
    }
}
