<?php

namespace App\Controllers\Api\Web;

use CodeIgniter\RESTful\ResourceController;
use App\Models\Api\Web\ProductsModel;
use App\Models\Api\Web\MarketsModel;
use App\Models\Api\Web\ServicesModel;
use App\Models\Api\Web\SellersModel;
use Exception;

class Home extends ResourceController
{

    protected $format = 'json';
    public $products;
    public $markets;
    public $services;
    public $sellers;

    public function __construct()
    {
        $this->products = new ProductsModel();
        $this->markets = new MarketsModel();
        $this->services = new ServicesModel();
        $this->sellers = new SellersModel();
    }
    // Search Districts
    // POST Request
    public function districtResults()
    {
    }

    // List Sellers
    // GET Request Type
    public function listSellers()
    {
        $response = [
            "status" => 200,
            "message" => "Sellers List",
            "error" => false,
            "data" => $this->sellers->listHomeSellers(),
        ];

        return $this->respondCreated($response);
    }

    // Seller Profile
    // GET Request Type
    public function sellerProfile($id)
    {
        $response = [
            "status" => 200,
            "message" => "Seller Profile",
            "error" => false,
            "data" => $this->sellers->sellerProfile($id),
        ];

        return $this->respondCreated($response);
    }

    // Seller Products
    public function sellerProducts($id)
    {
        $response = [
            "status" => 200,
            "message" => "Seller Products",
            "error" => false,
            "data" => $this->sellers->getSellerProducts($id),
        ];

        $this->respondCreated($response);
    }

    // List Markets
    // GET Request Type
    public function listMarkets()
    {
        $response = [
            "status" => 200,
            "message" => "Market List",
            "error" => false,
            "data" => $this->markets->listHomeMarkets(),
        ];

        return $this->respondCreated($response);
    }

    // Market Results
    // GET Request Type
    public function listMarketProducts($id = null)
    {
        $response = [
            "status" => 200,
            "message" => "Products List",
            "error" => false,
            "data" => $this->markets->listMarketProducts($id),
        ];

        return $this->respondCreated($response);
    }

    // List Products
    // GET Request Type
    public function listProducts()
    {
        $response = [
            "status" => 200,
            "message" => "Products List",
            "error" => false,
            "data" => $this->products->listHomeProducts(),
        ];

        return $this->respondCreated($response);
    }

    // Product Details
    public function productDetails($id)
    {
        $response = [
            "status" => 200,
            "message" => "Product Details",
            "error" => false,
            "data" => $this->products->productDetails($id),
        ];

        return $this->respondCreated($response);
    }

    // Product Reviews
    public function productReviews($id)
    {
        $response = [
            "status" => 200,
            "message" => "Product Reviews",
            "error" => false,
            "data" => $this->products->productReviews($id),
        ];

        return $this->respondCreated($response);
    }

    // List Services
    // GET Request Type
    public function listServices()
    {
        $response = [
            "status" => 200,
            "message" => "Service List",
            "error" => false,
            "data" => $this->services->listHomeServices(),
        ];

        return $this->respondCreated($response);
    }

    // Service details
    // GET request
    public function serviceDetails($id)
    {
        $response = [
            "status" => 200,
            "message" => "Service Details",
            "error" => false,
            "data" => $this->services->serviceDetails($id),
        ];

        return $this->respondCreated($response);
    }

    // Service Reviews
    public function serviceReviews($id)
    {
        try {
            $response = [
                "status" => 200,
                "message" => "Service Reviews",
                "error" => false,
                "data" => $this->services->serviceReviews($id),
            ];
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => false,
                "data" => [],
            ];
        }


        return $this->respondCreated($response);
    }
}
