<?php

namespace App\Controllers\Api\Web;

use CodeIgniter\RESTful\ResourceController;
use App\Controllers\BaseController;
use Myth\Auth\Config\Auth as AuthConfig;
use Myth\Auth\Entities\User;
use Myth\Auth\Models\UserModel;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use Exception;
use App\Models\Web\ProductOrdersModel;
use App\Models\Web\ServiceOrdersModel;
use App\Models\Web\ReportAbuseModel;
use App\Models\Web\UserRequestsModel;
use App\Models\Web\ProductReviewsModel;
use App\Models\Web\ServiceReviewsModel;
use App\Models\Api\Web\ProductsModel;
use App\Models\Api\Web\ServicesModel;
use App\Models\Api\Web\CustomersModel;

class Users extends ResourceController
{

    protected $auth;
    protected $config;
    protected $session;
    protected $format = 'json';
    public $product_orders;
    public $service_orders;
    public $complaints;
    public $custom_requests;
    public $review_product;
    public $review_service;
    public $products;
    public $services;
    public $customer;
    public $user;
    public $db;

    public function __construct()
    {
        $this->product_orders = new ProductOrdersModel();
        $this->service_orders = new ServiceOrdersModel();
        $this->complaints = new ReportAbuseModel();
        $this->custom_requests = new UserRequestsModel();
        $this->review_product = new ProductReviewsModel();
        $this->review_service = new ServiceReviewsModel();
        $this->products = new ProductsModel();
        $this->services = new ServicesModel();
        $this->customer = new CustomersModel();
        // Most services in this controller require
        // the session to be started - so fire it up!
        $this->session = service('session');

        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->user = new UserModel();
        $this->db = \Config\Database::connect();
    }

    // Customer profile
    public function profile()
    {
        $authorize = $this->request->getHeader("Authorization");

        try {

            if (isset($authorize)) {
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $district = $this->user->getUserDistrict($decoded_data->userdata->district);
                $market = $this->user->getUserMarket($decoded_data->userdata->market);

                if ($decoded_data->userdata->category == 2) {
                    $response = [
                        "status" => 200,
                        "message" => "Profile Data",
                        "error" => false,
                        "data" => [
                            "id" => $decoded_data->userdata->id,
                            "firstname" => $decoded_data->userdata->firstname,
                            "lastname" => $decoded_data->userdata->lastname,
                            "store_name" => $decoded_data->userdata->store_name,
                            "store_tagline" => $decoded_data->userdata->store_tagline,
                            "store_description" => $decoded_data->userdata->store_description,
                            "category" => $decoded_data->userdata->category,
                            "market" => $market->market_name,
                            "slug" => $decoded_data->userdata->slug,
                            "phone" => $decoded_data->userdata->phone,
                            "email" => $decoded_data->userdata->email,
                            "address" => $decoded_data->userdata->address,
                            "district" => $district->district_name,
                            "profile_image" => $decoded_data->userdata->profile_image,
                        ],
                    ];
                } else {

                    $response = [
                        "status" => 200,
                        "message" => "Profile Data",
                        "error" => false,
                        "data" => [
                            "id" => $decoded_data->userdata->id,
                            "firstname" => $decoded_data->userdata->firstname,
                            "lastname" => $decoded_data->userdata->lastname,
                            "category" => $decoded_data->userdata->category,
                            "dob" => $decoded_data->userdata->dob,
                            "phone" => $decoded_data->userdata->phone,
                            "email" => $decoded_data->userdata->email,
                            "address" => $decoded_data->userdata->address,
                            "district" => $district->district_name,
                            "profile_image" => $decoded_data->userdata->profile_image,
                        ],
                    ];
                }

                // $response = [
                //     "status" => 200,
                //     "message" => "Profile Data",
                //     "error" => false,
                //     "data" => [
                //         "user" => $decoded_data,
                //         "id" => $decoded_data->userdata->id,
                //     ],
                // ];
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    public function update_account()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                "firstname" => "required",
                "lastname" => "required",
                "phone" => "required|min_length[10]",
                "district" => "required",
                "address" => "required",
                "city" => "required",
            ];

            $messages = [
                'firstname' => [
                    'required' => 'First Name is required',
                ],
                'lastname' => [
                    'required' => 'Last Name is required',
                ],
                'phone' => [
                    'required' => 'Mobile is required',
                    'min_length' => 'Mobile cannot be less than 10 characters',
                ],
                'district' => [
                    'required' => 'Your district is required',
                ],
                'address' => [
                    'required' => 'Your address is required',
                ],
                'city' => [
                    'required' => 'Your city is required',
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $old_image = $decoded_data->userdata->profile_image;

                    if (!empty($_FILES['image']['name'])) {
                        $image = $this->request->getFile('image');
                        $new_image = $image->getRandomName();
                    }

                    $data = [
                        'firstname' => $this->request->getVar('firstname'),
                        'lastname' => $this->request->getVar('lastname'),
                        'phone' => $this->request->getVar('phone'),
                        'address' => $this->request->getVar('address'),
                        'district' => $this->request->getVar('district'),
                        'profile_image' => (!empty($_FILES['image']['name']) ? $new_image : $old_image),
                    ];

                    if ($this->user->update($userID, $data)) {
                        if (!empty($_FILES['image']['name'])) {
                            $image->move('public/uploads/users/profiles', $new_image);
                        }
                        // Success
                        $response = [
                            "status" => 200,
                            "message" => 'Account updated successfully',
                            "error" => false,
                            "data" => [],
                        ];
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Falied to update account',
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } catch (Exception $ex) {
                    //throw $th;
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Update Delivery address
    public function update_delivery()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                "billing_names" => "required",
                "billing_phone" => "required|min_length[10]",
                "billing_address" => "required",
                "billing_district" => "required",
                "billing_city" => "required",
            ];

            $messages = [
                'billing_names' => [
                    'required' => 'Billing names is required',
                ],
                'billing_phone' => [
                    'required' => 'Billing phone is required',
                    'min_length' => 'Mobile cannot be less than 10 characters',
                ],
                'billing_address' => [
                    'required' => 'Billing address is required',
                ],
                'billing_district' => [
                    'required' => 'Billing district is required',
                ],
                'billing_city' => [
                    'required' => 'Billing city is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $billing_names = $this->request->getVar('billing_names');
                    $billing_phone = $this->request->getVar('billing_phone');
                    $billing_address = $this->request->getVar('billing_address');
                    $billing_district = $this->request->getVar('billing_district');
                    $billing_city = $this->request->getVar('billing_city');

                    $query = $this->db->query("UPDATE users SET billing_names=" . $this->db->escape($billing_names) . ", billing_phone=" . $this->db->escape($billing_phone) . ", billing_address=" . $this->db->escape($billing_address) . ", billing_district=" . $this->db->escape($billing_district) . ", billing_city=" . $this->db->escape($billing_city) . " WHERE id='$userID'");

                    if ($query) {
                        // Success
                        $response = [
                            "status" => 200,
                            "message" => 'Billing info updated successfully',
                            "error" => false,
                            "data" => [],
                        ];
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Falied to update billing info',
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } catch (Exception $ex) {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    public function getKey()
    {
        $key = "ADcdGdecDEVNK";
        return $key;
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }

    // Place order
    public function placeOrder()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                "qty" => "required",
                "price" => "required",
                "vendor_id" => "required",
                "prod_id" => "required",
                "prod_units" => "required",
            ];

            $messages = [
                'qty' => [
                    'required' => 'Product Quantity is required',
                ],
                'price' => [
                    'required' => 'Product Price is required',
                ],
                'vendor_id' => [
                    'required' => 'Vendor ID is required',
                ],
                'prod_id' => [
                    'required' => 'Product ID is required',
                ],
                'prod_units' => [
                    'required' => 'Product Units is required',
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $prod_id = $this->request->getVar('prod_id');
                        $results = $this->product_orders->getProductData($prod_id);
                        $inv_ref = 'INV' . random_string('alnum', 7);
                        $qty = $this->request->getVar('qty');
                        $price = $this->request->getVar('price');
                        $total = $qty * $price;

                        $data = [
                            'order_invoice' => $inv_ref,
                            'user_id' => $userID,
                            'vendor_id' => $this->request->getVar('vendor_id'),
                            'prod_id' => $prod_id,
                            'prod_category' => $results->product_category,
                            'prod_market' => $results->product_market,
                            'prod_district' => $results->product_district,
                            'pro_order_unit' => $this->request->getVar('prod_units'),
                            'order_qty' => $qty,
                            'order_total' => $total,
                            'order_discount' => 0,
                            'payment_status' => 0,
                            'order_status' => 2,
                            'read_status' => 0,
                        ];

                        if ($this->product_orders->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Order placed successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to place order',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Place service order
    public function placeServiceOrder()
    {

        // Validate request type
        if ($this->request->getMethod() == 'post') {

            $rules = [
                "qty" => "required",
                "price" => "required",
                "vendor_id" => "required",
                "service_id" => "required",
                "service_unit" => "required",
            ];

            $messages = [
                'qty' => [
                    'required' => 'Service Quantity is required',
                ],
                'price' => [
                    'required' => 'Service Price is required',
                ],
                'vendor_id' => [
                    'required' => 'Vendor ID is required',
                ],
                'service_id' => [
                    'required' => 'Service ID is required',
                ],
                'service_unit' => [
                    'required' => 'Service Unit is required',
                ],
            ];


            if (!$this->validate($rules, $messages)) {

                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $service_id = $this->request->getVar('service_id');
                        $results = $this->service_orders->getServiceData($service_id);

                        $inv_ref = 'INV' . random_string('alnum', 7);
                        $qty = $this->request->getVar('qty');
                        $price = $this->request->getVar('price');
                        $total = $qty * $price;

                        $data = [
                            's_order_invoice' => $inv_ref,
                            's_user_id' => $userID,
                            's_vendor_id' => $this->request->getVar('vendor_id'),
                            's_service_id' => $service_id,
                            's_category' => $results->service_category,
                            's_district' => $results->service_district,
                            's_service_unit' => $this->request->getVar('service_unit'),
                            's_service_unit_price' => $this->request->getVar('price'),
                            's_order_qty' => $qty,
                            's_order_total' => $total,
                            's_order_discount' => 0,
                            's_payment_status' => 0,
                            's_order_status' => 2,
                            'read_status' => 0,
                        ];

                        if ($this->service_orders->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Order placed successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to place order',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Send Complaint
    public function send_complain()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'reason' => 'required',
                'description' => 'required|max_length[255]'
            ];

            $messages = [
                'reason' => [
                    'required' => 'Complaint Reason is required',
                ],
                'description' => [
                    'required' => 'Please describe your complain',
                    'max_length' => 'Your description cannot be more than 255 characters',
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $report_ref = 'INV' . random_string('alnum', 7);

                        $data = [
                            'report_ref' => $report_ref,
                            'reason' => $this->request->getVar('reason'),
                            'user_id' => $userID,
                            'vendor_id' => $this->request->getVar('vendor_id'),
                            'description' => $this->request->getVar('description'),
                            'read_status' => 0,
                        ];

                        if ($this->complaints->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Complaint sent successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to send complaint',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Send Custom Request
    public function send_custom_request()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'subject' => 'required|max_length[150]',
                'type' => 'required',
                'description' => 'required|max_length[255]'
            ];

            $messages = [
                'subject' => [
                    'required' => 'Please enter your subject',
                    'max_length' => 'Subject cannot be more than 150 characters'
                ],
                'type' => [
                    'required' => 'Please select request type',
                ],
                'description' => [
                    'required' => 'Request description is required',
                    'max_length' => 'Your description cannot be more 255 characters'
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $request_ref = 'INV' . random_string('alnum', 7);

                        $data = [
                            'request_ref' => $request_ref,
                            'user_id' => $userID,
                            'vendor_id' => $this->request->getVar('vendor_id'),
                            'request_title' => $this->request->getVar('subject'),
                            'request_type' => $this->request->getVar('type'),
                            'request_desc' => $this->request->getVar('description'),
                            'request_status' => 2,
                            'read_status' => 0,
                        ];

                        if ($this->custom_requests->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Request sent successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to send Request',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Review Product
    public function review_product()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'rating' => 'required',
                'title' => 'required|',
                'description' => 'required|max_length[255]'
            ];

            $messages = [
                'rating' => [
                    'required' => 'Please select rating',
                ],
                'title' => [
                    'required' => 'Rating title required',
                ],
                'description' => [
                    'required' => 'Rating description is required',
                    'max_length' => 'Rating description cannot be more 255 characters'
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $request_ref = 'INV' . random_string('alnum', 7);

                        $data = [
                            'reviewer_id' => $userID,
                            'r_product_vendor' => $this->request->getVar('vendor'),
                            'reviewed_product' => $this->request->getVar('product'),
                            'review_title' => ucwords($this->request->getVar('title')),
                            'review_desc' => $this->request->getVar('description'),
                            'review_rating' => $this->request->getVar('rating'),
                        ];

                        if ($this->review_product->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Review submitted successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to submit review',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Review Service
    public function review_service()
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'rating' => 'required',
                'title' => 'required|',
                'description' => 'required|max_length[255]'
            ];

            $messages = [
                'rating' => [
                    'required' => 'Please select rating',
                ],
                'title' => [
                    'required' => 'Rating title required',
                ],
                'description' => [
                    'required' => 'Rating description is required',
                    'max_length' => 'Rating description cannot be more 255 characters'
                ],
            ];


            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    helper('text');
                    // No errors
                    $authorize = $this->request->getHeader("Authorization");
                    $token = $authorize->getValue();
                    $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                    $userID = $decoded_data->userdata->id;
                    $user_category = $decoded_data->userdata->category;

                    // Deny admin accounts from placing orders
                    if ($user_category == 1) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } elseif ($user_category == 2) {
                        // Deny vendor accounts from placing orders
                        $response = [
                            "status" => 500,
                            "message" => 'Authorization Error !! Please register with a customer account to make orders',
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        $data = [
                            'reviewer_id' => $userID,
                            'r_service_vendor' => $this->request->getVar('vendor'),
                            'reviewed_service' => $this->request->getVar('service'),
                            'review_title' => ucwords($this->request->getVar('title')),
                            'review_desc' => $this->request->getVar('description'),
                            'review_rating' => $this->request->getVar('rating'),
                        ];

                        if ($this->review_service->insert($data)) {
                            // Success
                            $response = [
                                "status" => 200,
                                "message" => 'Review submitted successfully',
                                "error" => false,
                                "data" => [],
                            ];
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Falied to submit review',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // User product orders
    public function product_order()
    {

        try {
            $authorize = $this->request->getHeader("Authorization");
            $token = $authorize->getValue();
            $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));
            $userID = $decoded_data->userdata->id;

            $response = [
                "status" => 200,
                "message" => 'Order List',
                "error" => false,
                "data" => $this->products->userOrders($userID),
            ];
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // User Service Orders
    public function service_orders()
    {

        try {

            $authorize = $this->request->getHeader("Authorization");
            $token = $authorize->getValue();
            $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));
            $userID = $decoded_data->userdata->id;

            $response = [
                "status" => 200,
                "message" => 'Order List',
                "error" => false,
                "data" => $this->services->userOrders($userID),
            ];
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Requests
    public function order_requests()
    {

        try {

            $authorize = $this->request->getHeader("Authorization");
            $token = $authorize->getValue();
            $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));
            $userID = $decoded_data->userdata->id;

            $response = [
                "status" => 200,
                "message" => 'Request List',
                "error" => false,
                "data" => $this->customer->getRequests($userID),
            ];
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }
}
