<?php

namespace App\Controllers\Forum;

use App\Controllers\BaseController;
use App\Models\Forum\FarmerGroups;
use App\Models\Forum\JoinGroupModel;
use App\Models\Forum\Topics;
use App\Models\Forum\DiscussionsModel;

class Home extends BaseController
{
    public $groups;
    public $join_group;
    public $topics;
    public $discussions;
    public $auth;
    public $db;
    public $session;

    public function __construct()
    {
        $this->groups = new FarmerGroups();
        $this->join_group = new JoinGroupModel();
        $this->topics = new Topics();
        $this->discussions = new DiscussionsModel();
        $this->db = \Config\Database::connect();
        $this->auth = service('authentication');
        $this->session = session();
        $pager = \Config\Services::pager();
    }

    public function index()
    {
        $data['page_title'] = 'Forum';
        $data['topics'] = $this->topics->getAllTopics();
        return view('web/forum/home', $data);
    }

    // Get Group Topics
    public function group_topics($slug)
    {
        // Fetch group id from slug
        $query = $this->db->query("SELECT * FROM farm_groups WHERE group_slug='$slug'");
        $result = $query->getRow();
        $data['page_title'] = $result->group_name;
        $data['group_id'] = $result->group_id;
        $data['slug'] = $result->group_slug;
        $data['group_topics'] = $this->groups->getGroupTopics($result->group_id);
        $data['db'] = $this->db;
        // Send Post Data
        if ($this->request->getMethod() == "post") {

            $user_id = $this->request->getVar('user_id');
            $group_id = $this->request->getVar('group_id');
            $group_name = $this->request->getVar('group_name');

            $data = [
                'join_user_id' => $user_id,
                'join_group_id' => $group_id,
            ];

            $save = $this->join_group->save($data);
            if ($save) {
                $this->session->setTempdata('success', 'Welcome to: ' . $group_name, 3);
                return redirect()->to(current_url());
            } else {
                $this->session->setTempdata('error', 'Sorry! Joining group failed', 3);
                return redirect()->to(current_url());
            }
        }
        return view('web/forum/group_topics', $data);
    }

    // Create Topic
    public function create_topic()
    {

        if ($this->request->getMethod() == 'post') {
            $slug = $this->request->getVar('slug');

            $rules = [
                'title' => 'required|is_unique[forum_topics.topic_title]',
                'description' => 'required'
            ];

            $messages = [
                'title' => [
                    'required' => 'Topic title is required',
                    'is_unique' => 'Please create a uniqu topic title'
                ],
                'description' => [
                    'required' => 'Topic description is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $this->session->setTempdata('error', 'Erros! There are errors in your form', 3);
                return redirect()->to(site_url('forum/topics/group/' . $slug));
            } else {

                $data = [
                    'topic_group_id' => $this->request->getVar('group_id'),
                    'topic_title' => $this->request->getVar('title'),
                    'topic_slug' => url_title(strtolower($this->request->getVar('title')), '-', true),
                    'topic_description' => $this->request->getVar('description'),
                    'topic_author' => $this->request->getVar('author_id'),
                ];

                $result = $this->topics->save($data);
                if ($result) {
                    $this->session->setTempdata('success', 'Topic Created Successfully', 3);
                    return redirect()->to(site_url('forum/topics/group/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Creating topic failed', 3);
                    return redirect()->to(site_url('forum/topics/group/' . $slug));
                }
            }
        }
    }

    // Tpoic Discussions
    public function topic_discussions($slug)
    {
        // Fetch Topic id from slug
        $query = $this->db->query("SELECT * FROM forum_topics INNER JOIN users ON users.id=forum_topics.topic_author INNER JOIN farm_groups ON farm_groups.group_id=forum_topics.topic_group_id WHERE topic_slug='$slug'");
        $result = $query->getRow();
        $data['page_title'] =  $result->topic_title;
        $data['topic'] = $query->getRow();
        $data['db'] = $this->db;
        $data['discussions'] = $this->topics->getTopicDiscussions($result->topic_id);
        if ($this->request->getMethod() == "post") {

            $topic_id = $this->request->getVar('topic_id');
            $farmer_id = $this->request->getVar('farmer_id');
            $group_id = $this->request->getVar('group_id');

            $data = [
                'discussion_topic_id' => $topic_id,
                'discussion' => $this->request->getVar('description'),
                'farmer_id' => $farmer_id,
                'discussion_group_id' => $group_id,
            ];

            $save = $this->discussions->save($data);

            if ($save) {
                $this->session->setTempdata('success', 'Reply added successful', 3);
                return redirect()->to(current_url());
            } else {
                $this->session->setTempdata('error', 'Sorry! sending reply failed', 3);
                return redirect()->to(current_url());
            }
        }
        return view('web/forum/topic_discussions', $data);
    }
}
