<?php

namespace App\Controllers\Seller;

use App\Controllers\BaseController;
use App\Models\Vendor\ServicesModel;
use App\Models\Admin\ServiceTypesModel;
use App\Models\Admin\ServiceUnitsModel;

class Services extends BaseController
{
    public $categories;
    public $services;
    public $units;
    public $session;

    public function __construct()
    {
        $this->categories = new ServiceTypesModel();
        $this->services = new ServicesModel();
        $this->units = new ServiceUnitsModel();
        $this->session = session();
    }

    public function index()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Services';
        $data['services'] = $this->services->getServices($user_id);
        return view('vendors/services/service_list', $data);
    }

    // Banned Services
    public function  banned_services()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Banned Services';
        $data['services'] = $this->services->getBannedServices($user_id);
        return view('vendors/services/banned_services', $data);
    }

    // Add products
    public function add_services()
    {
        $data['breadcrumb'] = 'Add Services';
        $data['categories'] = $this->categories->getServiceTypes();
        $data['service_units'] = $this->units->getServiceUnits();
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
                'keywords' => 'required',
                'meta_desc' => 'required',
                'category'  => 'required',
                'units'  => 'required',
                'price' => 'required',
                'description' => 'required',
                'service_image' => 'uploaded[service_image]|is_image[service_image]|mime_in[service_image,image/jpg,image/jpeg,image/gif,image/png,image/webp]|max_size[service_image,2054]',
            ];

            $messages = [
                'name' => [
                    'required' => 'Service name is required'
                ],
                'keywords' => [
                    'required' => 'Service keywords is required'
                ],
                'meta_desc' => [
                    'required' => 'Service meta description is required'
                ],
                'description' => [
                    'required' => 'Service description is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                if (!empty($_FILES['service_image']['name'])) {
                    $image = $this->request->getFile('service_image');
                    $new_image = $image->getRandomName();
                }

                $service_ref = random_string('alnum', 10);
                $sku = "AG" . random_string('alnum', 5);

                $data = [
                    'service_ref' => $service_ref,
                    'service_category' => $this->request->getVar('category'),
                    'service_vendor' => $this->request->getVar('vendor'),
                    'service_name' => ucwords($this->request->getVar('name')),
                    'service_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'service_keywords' => strtolower($this->request->getVar('keywords')),
                    'service_meta_desc' => ucwords($this->request->getVar('meta_desc')),
                    'service_desc' => $this->request->getVar('description'),
                    'service_image' => $new_image,
                    'service_unit' => $this->request->getVar('units'),
                    'sku' => $sku,
                    'service_price' => $this->request->getVar('price'),
                    'service_district' => user()->district,
                    'service_status' => 1,
                ];

                $result = $this->services->save($data);

                if ($result) {
                    if (!empty($_FILES['service_image']['name'])) {
                        $image->move('public/uploads/services', $new_image);
                    }
                    $this->session->setTempdata('success', 'Service added successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Service could not be added, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('vendors/services/add_service', $data);
    }

    // Edit Products
    public function edit_service($id)
    {
        $data['breadcrumb'] = 'Edit Service';
        $data['categories'] = $this->categories->getServiceTypes();
        $data['service_units'] = $this->units->getServiceUnits();
        $data['service'] = $this->services->getServiceID($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
                'keywords' => 'required',
                'meta_desc' => 'required',
                'category'  => 'required',
                'units'  => 'required',
                'price' => 'required',
                'description' => 'required'
            ];

            $messages = [
                'name' => [
                    'required' => 'Service name is required'
                ],
                'keywords' => [
                    'required' => 'Service keywords is required'
                ],
                'meta_desc' => [
                    'required' => 'Service meta description is required'
                ],
                'description' => [
                    'required' => 'Service description is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['service_image']['name'])) {
                    // Unlink old image
                    $findFile = $this->services->getServiceID($id);
                    $old_img = $findFile->service_image;
                    $imagePath = 'public/uploads/services/' . $old_img;

                    if (file_exists($imagePath)) {
                        unlink('public/uploads/services/' . $old_img);
                    }

                    $image = $this->request->getFile('service_image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'service_category' => $this->request->getVar('category'),
                    'service_vendor' => $this->request->getVar('vendor'),
                    'service_name' => ucwords($this->request->getVar('name')),
                    'service_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'service_keywords' => strtolower($this->request->getVar('keywords')),
                    'service_meta_desc' => ucwords($this->request->getVar('meta_desc')),
                    'service_desc' => $this->request->getVar('description'),
                    'service_image' => (!empty($_FILES['service_image']['name']) ? $new_image : $this->request->getVar('old_image')),
                    'service_unit' => $this->request->getVar('units'),
                    'service_price' => $this->request->getVar('price'),
                ];

                $result = $this->services->update($id, $data);

                if ($result) {

                    if (!empty($_FILES['service_image']['name'])) {
                        $image->move('public/uploads/services', $new_image);
                    }
                    $this->session->setTempdata('success', 'Service updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Service could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('vendors/services/edit_service', $data);
    }

    // Delete product
    public function delete_service($id)
    {
        // Unlink old image
        $findFile = $this->services->getServiceID($id);
        $old_img = $findFile->service_image;
        $imagePath = 'public/uploads/services/' . $old_img;

        if (file_exists($imagePath)) {
            unlink('public/uploads/services/' . $old_img);
        }

        $result = $this->services->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Services Deleted successfully', 3);
            return redirect()->to(site_url('seller/services'));
        }
    }
}
