<?php

namespace App\Controllers\Web;

use App\Controllers\BaseController;
use App\Models\Web\UserModel;
use App\Models\Web\UserRequestsModel;
use Myth\Auth\Config\Auth as AuthConfig;
use Myth\Auth\Entities\User;

class Users extends BaseController
{
    protected $auth;
    protected $config;
    public $users;
    public $requests;
    protected $session;
    public $db;

    public function __construct()
    {
        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->users = new UserModel();
        $this->requests = new UserRequestsModel();
        $this->db = \Config\Database::connect();
        $this->session = session();
    }

    // Dashboard
    public function dashboard()
    {
        $user_id = user()->id;
        $data['successful_orders'] = $this->users->getSuccessfullOrders($user_id);
        $data['pending_orders'] = $this->users->getPendingOrders($user_id);
        $data['cancelled_orders'] = $this->users->getCancelledOrders($user_id);
        $data['orders'] = $this->users->getOrders($user_id);
        return view('web/users/dashboard', $data);
    }

    public function order_history()
    {
        $user_id = user()->id;
        $data['orders'] = $this->users->getOrders($user_id);
        $data['services'] = $this->users->getServices($user_id);
        return view('web/users/order_history', $data);
    }

    public function settings()
    {
        $user_id = user()->id;
        $data['user_data'] = $this->users->getUserSettings($user_id);
        $data['districts'] = $this->users->getDistricts();
        return view('web/users/settings', $data);
    }

    // Requests
    public function requests()
    {
        $user_id = user()->id;
        $data['requests'] = $this->requests->getRequests($user_id);
        return view('web/users/requests', $data);
    }
    public function request_details($id)
    {
        $data['request'] = $this->requests->getRequestDetails($id);
        return view('web/users/request_details', $data);
    }

    // Edit Billing
    public function edit_billing_info($id)
    {

        $user_id = user()->id;
        $data['user_data'] = $this->users->getUserSettings($user_id);
        $data['districts'] = $this->users->getDistricts();

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'billing_names' => 'required',
                'billing_number' => 'required|min_length[10]',
                'billing_city' => 'required',
                'billing_address' => 'required',
            ];

            $messages = [
                'billing_names' => [
                    'required' => 'Billing names are required'
                ],
                'billing_number' => [
                    'required' => 'Billing number is required',
                    'min_length' => 'Phone number cannot be less than 10 characters',
                ],
                'billing_city' => [
                    'required' => 'Billing city is required'
                ],
                'billing_address' => [
                    'required' => 'Billing address is required'
                ],

            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $data = [
                    'billing_names' => ucwords($this->request->getVar('billing_names')),
                    'billing_address' => $this->request->getVar('billing_address'),
                    'billing_phone' => $this->request->getVar('billing_number'),
                    'billing_city' => $this->request->getVar('billing_city'),
                ];

                $result = $this->users->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Billing info updated successfully', 3);
                    return redirect()->to('user/settings');
                } else {
                    $this->session->setTempdata('erro', 'Billing info couldnot be updated', 3);
                    return redirect()->to('user/settings');
                }
            }
        }
        return view('web/users/settings', $data);
    }

    public function edit_account($id)
    {
        $user_id = user()->id;
        $data['user_data'] = $this->users->getUserSettings($user_id);
        $data['districts'] = $this->users->getDistricts();

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'acc_phone' => 'required',
                'acc_district' => 'required'
            ];

            if (!empty($_FILES['image']['name'])) {
                $rules = [
                    'image' => 'uploaded[image]|max_size[image,2054]|is_image[image]',
                ];
            }

            $messages = [
                'acc_phone' => [
                    'required' => 'Phone number is required'
                ],
                'acc_district' => [
                    'required' => 'District info is required',
                ]
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $old_img = user()->profile_image;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/users/profiles/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'phone' => $this->request->getVar('acc_phone'),
                    'district' => $this->request->getVar('acc_district'),
                    'profile_image' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];

                $result = $this->users->update($id, $data);
                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/users/profiles', $new_image);
                    }
                    $this->session->setTempdata('success', 'Account info updated successfully', 3);
                    return redirect()->to('user/settings');
                } else {
                    $this->session->setTempdata('erro', 'Account info couldnot be updated', 3);
                    return redirect()->to('user/settings');
                }
            }
        }
        return view('web/users/settings', $data);
    }

    public function edit_password($id)
    {
        $user_id = user()->id;

        $data['user_data'] = $this->users->getUserSettings($user_id);
        $data['districts'] = $this->users->getDistricts();

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'password' => 'required|min_length[8]',
                'c_password' => 'required|matches[password]',
            ];

            $messages = [
                'password' => [
                    'required' => 'Password is required'
                ],
                'c_password' => [
                    'required' => 'Confirm password field is required',
                    'matches' => 'The two passwords donot match',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $config = config('Auth');
                if (
                    (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                    ||
                    (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
                ) {
                    $hashOptions = [
                        'memory_cost' => $config->hashMemoryCost,
                        'time_cost'   => $config->hashTimeCost,
                        'threads'     => $config->hashThreads,
                    ];
                } else {
                    $hashOptions = [
                        'cost' => $config->hashCost,
                    ];
                }

                $password = password_hash(self::preparePassword($this->request->getVar('password')), $this->config->hashAlgorithm, $hashOptions);

                $data = [
                    'password_hash' => $password,
                ];


                $result = $this->users->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Password updated successfully', 3);
                    return redirect()->to('user/settings');
                } else {
                    $this->session->setTempdata('erro', 'Password couldnot be updated', 3);
                    return redirect()->to('user/settings');
                }
            }
        }
        return view('web/users/settings', $data);
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }

    public function addresses()
    {
        $user_id = user()->id;
        $data['user_data'] = $this->users->getUserSettings($user_id);
        return view('web/users/addresses', $data);
    }

    // User order invoice
    public function order_invoice($invoice_id)
    {
        $data['orders'] = $this->users->getOrderInvoice($invoice_id);
        return view('web/users/order_invoice', $data);
    }
    // User service invoice
    public function service_invoice($invoice_id)
    {
        $data['orders'] = $this->users->getServiceInvoice($invoice_id);
        return view('web/users/service_invoice', $data);
    }
}
