<?php

namespace App\Models\Admin;

use CodeIgniter\Model;

class HomeModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = '';

    // Total Orders
    public function countOrderSales()
    {
        $orders = $this->db->table('orders');
        $orders->where(['order_status' => 1]);
        $result = $orders->countAllResults();

        $services = $this->db->table('service_orders');
        $services->where(['s_order_status' => 1]);
        $result2 = $services->countAllResults();

        return $result + $result2;
    }

    // Pending Orders
    public function countPendingOrders()
    {
        $orders = $this->db->table('orders');
        $orders->where(['order_status' => 2]);
        $result = $orders->countAllResults();

        $services = $this->db->table('service_orders');
        $services->where(['s_order_status' => 2]);
        $result2 = $services->countAllResults();

        return $result + $result2;
    }

    // Cancelled Orders
    public function countCancelledOrders()
    {
        $orders = $this->db->table('orders');
        $orders->where(['order_status' => 0]);
        $result = $orders->countAllResults();

        $services = $this->db->table('service_orders');
        $services->where(['s_order_status' => 0]);
        $result2 = $services->countAllResults();

        return $result + $result2;
    }

    // Total Products
    public function countTotalProducts()
    {
        $builder = $this->db->table('products');
        $builder->where(['product_status' => 1]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Total Services
    public function countTotalServices()
    {
        $builder = $this->db->table('services');
        $builder->where(['service_status' => 1]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Approved Sellers
    public function countApprovedSellers()
    {
        $builder = $this->db->table('users');
        $builder->where(['category' => 2, 'verification' => 1]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Banned Sellers
    public function countBannedSellers()
    {
        $builder = $this->db->table('users');
        $builder->where(['category' => 2, 'verification' => 0]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Customers
    public function countCustomers()
    {
        $builder = $this->db->table('users');
        $builder->where(['category' => 3]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Sum Total Sales
    public function sumApprovedSales()
    {
        $orders = $this->db->table('orders');
        $orders->selectSum('order_total');
        $orders->where(['order_status' => 1]);
        $result = $orders->get();

        $services = $this->db->table('service_orders');
        $services->selectSum('s_order_total');
        $services->where(['s_order_status' => 1]);
        $result2 = $services->get();

        return $result + $result2;
    }

    // Count Requests
    public function countRequests()
    {
        $builder = $this->db->table('product_requests');
        $result = $builder->countAllResults();
        return $result;
    }

    // Revenue
    public function revenue()
    {
        // $query = $this->db->query("SELECT SUM('order_total') as total FROM orders");
        // $result = $query->getRow()->total;

        $builder = $this->db->table('orders');
        $builder->selectSum('order_total');
        $builder->where(['order_status' => 1]);
        return $builder->get()->getRow();

    }

    public function pendingRevenue(){
        $builder = $this->db->table('orders');
        $builder->selectSum('order_total');
        $builder->where(['order_status' => 2]);
        return $builder->get()->getRow();
    }
    public function cancelledRevenue(){
        $builder = $this->db->table('orders');
        $builder->selectSum('order_total');
        $builder->where(['order_status' => 0]);
        return $builder->get()->getRow();
    }

    public function service_revenue(){
        $builder = $this->db->table('service_orders');
        $builder->selectSum('s_order_total');
        $builder->where(['s_order_status' => 1]);
        return $builder->get()->getRow();
    }

    public function pendingServiceRevenue(){
        $builder = $this->db->table('service_orders');
        $builder->selectSum('s_order_total');
        $builder->where(['s_order_status' => 2]);
        return $builder->get()->getRow();
    }
    public function cancelledServiceRevenue(){
        $builder = $this->db->table('service_orders');
        $builder->selectSum('s_order_total');
        $builder->where(['s_order_status' => 0]);
        return $builder->get()->getRow();
    }

    // Get recent products
    public function getProducts()
    {
        $builder = $this->db->table('products');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get last date sales
    public function lastDateRevenue(){
        $query = $this->db->query("SELECT * FROM employees  WHERE DATE(order_created) = DATE(NOW()) ORDER BY `id` DESC");
    }

    // Get last 7 days sales
    // If you want to get the last 10 days or the last 15 days records from a database table, you can change the query accordingly.
    public function last7DaysRevenue(){
        $query = $this->db->query("SELECT name, order_created FROM employees WHERE order_created >= DATE_SUB(DATE(NOW()), INTERVAL 7 DAY) ORDER BY order_created DESC");
    }

    // Fetch Day Wise LastWeek Data
    // Fetch day-wise last week data from the mysql database table.
    public function dayWiseRevenue(){
        $query = $this->db->query("SELECT SUM('order_total') as total FROM orders WHERE date(order_created) < DATE_SUB(NOW(), INTERVAL 1 WEEK) AND MONTH(order_created) = MONTH(CURDATE()) AND YEAR(order_created) = YEAR(CURDATE()) GROUP BY DAYNAME(order_created) ORDER BY (order_created)");
        return $query->getRow()->total;
    }

    // Get last week's sales
    public function lastWeekRevenue(){
        $query = $this->db->query("SELECT SUM('order_total') as total FROM orders WHERE YEARWEEK(`order_created`, 1) = YEARWEEK( CURDATE() - INTERVAL 1 WEEK, 1)");
        return $query->getRow();
    }

    // Get last month Sales
    public function lastMonthRevenue()
    {
        $builder = $this->db->table('orders');
        $builder->selectSum('order_total');
        $builder->where(['YEAR(order_created)' => 'YEAR(CURRENT_DATE - INTERVAL 1 MONTH)', 'MONTH(order_created)' => 'MONTH(CURRENT_DATE - INTERVAL 1 MONTH)']);
        return $builder->get()->getRow();

        // $query = $this->db->query("SELECT SUM('order_total') as total FROM orders WHERE YEAR(order_created) = YEAR(CURRENT_DATE - INTERVAL 1 MONTH) AND MONTH(order_created) = MONTH(CURRENT_DATE - INTERVAL 1 MONTH)");
        // return $query->getRow()->total;
    }

    // Get Last 3 Month Record
    // If you want to get last 3 month or 6-month records from the database table, you can change the query accordingly.
    public function last3MonthRevenue(){
        $query = $this->db->query("SELECT name, order_created FROM employees WHERE order_created >= DATE(NOW()) - INTERVAL 3 MONTH");
    }

    // Fetch Month Wise Last Year Data
    // Fetch date wise last month’s records from the MySQL database table.
    public function DayMonthWiseRevenue(){
        $query = $this->db->query("SELECT COUNT(id) as Count, DAY(order_created) as 'Day', DAYNAME(order_created) as 'Day Name', MONTHNAME(order_created) as 'Month Name' FROM employees WHERE MONTH(order_created) = MONTH(CURDATE() - INTERVAL 1 MONTH) AND YEAR(order_created) = YEAR(CURDATE())
        GROUP BY DAY(order_created)");
    }

    // Fetch Last Year Record
    // You want to fetch the last year records from the mysql database table.
    public function lastYearRecords(){
        $query = $this->db->query("SELECT name, created_at, YEAR(created_at) as year FROM employees 
        WHERE YEAR(created_at) = YEAR(NOW() - INTERVAL 1 YEAR) ORDER BY `id` DESC");
    }

    // Year Wise Data
    // You want to fetch year-wise data. Use the below query for getting year-wise data from the mysql database table.
    public function yearWiseRevenue(){
        $query = $this->db->query("SELECT COUNT(id) as Count,YEAR(created_at) as Year FROM employees
        GROUP BY YEAR(created_at)");
    }
}
