<?php

namespace App\Models\Api\Web;

use CodeIgniter\Model;

class ProductsModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'products';
    protected $primaryKey       = 'product_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Home Products
    public function listHomeProducts()
    {
        $builder = $this->db->table('products');
        $builder->select('products.*, categories.category_name as category, users.store_name as seller');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get(24)->getResult();
        return $result;
    }

    // Product Details
    public function productDetails($id)
    {
        $builder = $this->db->table('products');
        $builder->select('products.*, categories.category_name as category, users.store_name as seller, users.address as address');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where('product_id', $id);
        $result = $builder->get(24)->getRow();
        return $result;
    }

    // Product Reviews
    public function productReviews($id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->select('product_reviews.*, users.firstname as firstname, users.lastname as lastname, users.profile_image as profileimage');
        $builder->join('users', 'users.id=product_reviews.reviewer_id');
        $builder->where('reviewed_product', $id);
        $result = $builder->get()->getRow();
        return $result;
    }

    // User orders
    public function userOrders($id)
    {
        $builder = $this->db->table('orders');
        $builder->select('orders.*, products.product_name as product');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['user_id' => $id]);
        $builder->orderBy('order_id', 'DESC');
        return $builder->get()->getResult();
    }

}
