<?php

namespace App\Models\Api\Web;

use CodeIgniter\Model;

class ServicesModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'services';
    protected $primaryKey       = 'service_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Home Services
    public function listHomeServices()
    {
        $builder = $this->db->table('services');
        $builder->select('services.*, service_types.service_type_name as category, users.store_name as seller');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->orderBy('service_id', 'DESC');
        $result = $builder->get(24)->getResult();
        return $result;
    }

    // Service Details
    public function serviceDetails($id)
    {
        $builder = $this->db->table('services');
        $builder->select('services.*, service_types.service_type_name as category, users.store_name as seller, users.address as address');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->where('service_id', $id);
        $result = $builder->get(24)->getRow();
        return $result;
    }

    // Service Reviews
    public function serviceReviews($id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->select('service_reviews.*, users.firstname as firstname, users.lastname as lastname, users.profile_image as profileimage');
        $builder->join('users', 'users.id=service_reviews.reviewer_id');
        $builder->where('reviewed_service', $id);
        $result = $builder->get()->getRow();
        return $result;
    }

    // User orders
    public function userOrders($id)
    {
        $builder = $this->db->table('service_orders');
        $builder->select('service_orders.*, services.service_name as service');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_user_id' => $id]);
        $builder->orderBy('s_order_id', 'DESC');
        return $builder->get()->getResult();
    }
}
