<?php

namespace App\Models\Vendor;

use CodeIgniter\Model;

class ProductOrdersModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'orders';
    protected $primaryKey       = 'order_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['order_invoice', 'user_id', 'vendor_id', 'prod_id', 'order_qty', 'order_total', 'order_discount', 'payment_status', 'order_status'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'order_created';
    protected $updatedField  = 'order_updated';

    // Get seller orders
    public function getProductOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->where(['vendor_id' => $user_id]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get Completed Orders
    public function getCompletedtOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 1, 'payment_status' => 1]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Count completed orders
    public function countCompletedOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 1, 'payment_status' => 1]);
        return $builder->countAllResults();
    }

    // Get Pending Orders
    public function getPendingOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 2, 'payment_status' => 0]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
    // Count pending orders
    public function countPendingOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 2, 'payment_status' => 0]);
        return $builder->countAllResults();
    }

    // Get Cancelled Orders
    public function getCancelledOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 0, 'payment_status' => 0]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
    // Count cancelled orders
    public function countCancelledOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 0, 'payment_status' => 0]);
        return $builder->countAllResults();
    }

    // Get order invoice
    public function getOrderInvoice($order_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_id' => $order_id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get recent orders
    public function getRecentOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->where(['vendor_id' => $user_id]);
        $builder->orderBy('order_id', 'DESC');
        return $builder->get(5)->getResult();
    }

    // Sum orders
    // public function sumOrders($user_id){
    //     $builder = $this->db->table('orders');
    //     $builder->select('sum(order_total) as total')->first();
    //     $builder->where(['vendor_id'=>$user_id, 'order_status'=>1, 'payment_status'=>1]);
    //     return $builder->first();
    // }

    // Get unread orders
    public function getUnreadOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->where(['vendor_id' => $user_id, 'order_status' => 2, 'payment_status' => 0]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
}
