<?php

namespace App\Models\Vendor;

use CodeIgniter\Model;

class ProductsModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'products';
    protected $primaryKey       = 'product_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['product_ref', 'product_category', 'product_vendor', 'product_name', 'product_slug', 'product_keywords', 'product_meta_desc', 'product_desc', 'product_image', 'product_attribute', 'product_unit', 'sale_type', 'unit_value', 'sku', 'product_price', 'product_discount', 'product_status', 'product_district', 'product_market'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    public function getProducts($user_id)
    {
        $builder = $this->db->table('products');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_vendor' => $user_id]);
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch product by id
    public function getProductID($id)
    {
        $builder = $this->db->table('products');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Fetch Banned Products
    public function getBannedProducts($user_id)
    {
        $builder = $this->db->table('products');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_status' => 0, 'product_vendor' => $user_id]);
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get recent products
    public function getRecentProducts($user_id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_vendor' => $user_id]);
        $builder->orderBy('product_id', 'DESC');
        return $builder->get(5)->getResult();
    }
}
