<?php

namespace App\Models\Vendor;

use CodeIgniter\Model;

class ServiceOrdersModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'service_orders';
    protected $primaryKey       = 's_order_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['s_order_invoice', 's_user_id', 's_vendor_id', 's_service_id', 's_order_qty', 's_order_total', 's_order_discount', 's_payment_status', 's_order_status'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 's_order_created';
    protected $updatedField  = 's_order_updated';

    // Get seller orders
    public function getServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->where(['s_vendor_id' => $user_id]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get Completed Orders
    public function getCompletedServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 1, 's_payment_status' => 1]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
    // Count completed service orders
    public function countCompletedServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 1, 's_payment_status' => 1]);
        return $builder->countAllResults();
    }

    // Get Pending Orders
    public function getPendingServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 2, 's_payment_status' => 0]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
    // Count pending service orders
    public function countPendingServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 2, 's_payment_status' => 0]);
        return $builder->countAllResults();
    }

    // Get Cancelled Orders
    public function getCancelledServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 0, 's_payment_status' => 0]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
    // Count cancelled service orders
    public function countCancelledServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 0, 's_payment_status' => 0]);
        return $builder->countAllResults();
    }

    // Get order invoice
    public function getServiceOrderInvoice($order_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_id' => $order_id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get recent orders
    public function getRecentOrders($user_id){
        $builder = $this->db->table('service_orders');
        $builder->where(['s_vendor_id' => $user_id]);
        $builder->orderBy('s_order_id', 'DESC');
        return $builder->get(5)->getResult();
    }

    // Sum orders
    public function sumOrders($user_id){
        $builder = $this->db->table('service_orders');
        $builder->selectSum('s_order_total');
        $builder->where(['s_vendor_id' => $user_id, 's_order_status' => 1, 's_payment_status' => 1]);
        return $builder->get();
    }
}
