<?php

namespace App\Models\Web;

use CodeIgniter\Model;

class ProductsModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'products';
    protected $primaryKey       = 'product_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    // Get product details
    public function getProductDetails($slug)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['product_slug' => $slug]);
        return $builder->get()->getRow();
    }

    // Get product reviews
    public function getProductReviews($id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->join('users', 'users.id=product_reviews.reviewer_id');
        $builder->where(['reviewed_product' => $id]);
        return $builder->get()->getResult();
    }

    // Fetch Related products
    public function getRelatedProduct($cat_id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['product_category' => $cat_id]);
        return $builder->get(4)->getResult();
    }

    // Count Product Reviews
    public function countReviews($product_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['reviewed_product' => $product_id]);
        return $builder->countAllResults();
    }

    // Fetch district products
    public function getProductsByDistrict($cat_id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['district' => $cat_id]);
        return $builder->get()->getResult();
    }

    // Fetch all products
    public function getAllProducts()
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->orderBy('product_id', 'DESC');
        return $builder->get()->getResult();
    }

    // Fetch recent products
    public function getRecentProducts()
    {
        $builder = $this->db->table('products');
        $builder->orderBy('product_id', 'DESC');
        return $builder->get(3)->getResult();
    }

    // Product categories
    public function getCategories()
    {
        $builder = $this->db->table('categories');
        $builder->orderBy('category_id', 'DESC');
        return $builder->get()->getResult();
    }

    // Fetch products by market
    public function getProductsByMarket($market_id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['product_market' => $market_id]);
        $builder->orderBy('product_id', 'DESC');
        return $builder->get()->getResult();
    }

    // Fetch markets by district
    public function getMarketsByDistrict($district_id)
    {
        $builder = $this->db->table('markets');
        $builder->join('districts', 'districts.district_id=markets.market_district');
        $builder->where(['market_district' => $district_id]);
        $builder->orderBy('market_id', 'DESC');
        return $builder->get()->getResult();
    }
}
