<?= $this->extend('vendors/layouts/app.php'); ?>

<?= $this->section('page_title'); ?>
Edit Product
<?= $this->endSection() ?>

<?= $this->section('body'); ?>
<div id="main-content">
    <div class="container-fluid">
        <div class="block-header">
            <div class="row clearfix">
                <div class="col-md-6 col-sm-12">
                    <h1>AMIS Uganda</h1>
                    <nav aria-label="breadcrumb">
                        <ol class="breadcrumb">
                            <li class="breadcrumb-item"><a href="#">Home</a></li>
                            <li class="breadcrumb-item"><a href="#">Page</a></li>
                            <li class="breadcrumb-item active" aria-current="page"><?= $breadcrumb; ?></li>
                        </ol>
                    </nav>
                </div>
                <div class="col-md-6 col-sm-12 text-right hidden-xs">
                    <a href="<?= site_url('seller/products') ?>" class="btn btn-sm btn-success"> Product List</a>
                </div>
            </div>
        </div>
        <div class="row clearfix">
            <div class="col-md-12">
                <div class="card">
                    <?php if (isset($validation)) : ?>
                        <div class="alert alert-danger" role="alert">
                            <?= $validation->listErrors() ?>
                        </div>
                    <?php endif; ?>

                    <?php if (session()->getTempdata('error')) : ?>
                        <div class="alert alert-danger alert-dismissible" role="alert">
                            <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                            <i class="fa fa-times-circle"></i> <?= session()->getTempdata('error'); ?>
                        </div>
                    <?php endif; ?>

                    <div class="body">
                        <form id="basic-form" method="post" action="<?= site_url('seller/edit_product/' . esc($product->product_id)) ?>" enctype="multipart/form-data" novalidate>
                            <?= csrf_field() ?>
                            <input type="text" name="vendor" class="form-control" value="<?= user()->id ?>" hidden>
                            <div class="form-group">
                                <label>Product Name <span class="text-danger">*</span></label>
                                <input type="text" name="name" class="form-control" value="<?= esc($product->product_name) ?>" required>
                            </div>
                            <div class="form-group">
                                <label>Product Slug <span class="text-success">(Auto generated)</span></label>
                                <input type="text" name="slug" class="form-control" value="<?= esc($product->product_slug) ?>" readonly>
                            </div>
                            <div class="form-group">
                                <label>Meta Keywords <span class="text-danger">*</span></label>
                                <input type="text" name="keywords" class="form-control" value="<?= esc($product->product_keywords) ?>" required>
                                <small class="text-success">To be used for SEO</small>
                            </div>
                            <div class="form-group">
                                <label>Meta Description <span class="text-danger">*</span></label>
                                <textarea name="meta_desc" class="form-control" rows="3" required><?= esc($product->product_meta_desc) ?></textarea>
                                <small class="text-success">To be used for SEO</small>
                            </div>
                            <div class="row">
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Product Category <span class="text-danger">*</span></label>
                                        <select class="custom-select" name="category" required>
                                            <option value="<?= esc($product->product_category) ?>" selected><?= esc($product->category_name) ?></option>
                                            <?php foreach ($categories as $category) : ?>
                                                <?php if ($product->product_category != $category->category_id) : ?>
                                                    <option value="<?= $category->category_id ?>"><?= $category->category_name ?></option>
                                                <?php endif; ?>
                                            <?php endforeach; ?>
                                        </select>
                                    </div>
                                </div>
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Units <span class="text-danger">*</span></label>
                                        <select class="custom-select" name="units" required>
                                            <option value="<?= esc($product->product_unit) ?>" selected><?= esc($product->unit_name) ?></option>
                                            <?php foreach ($units as $unit) : ?>
                                                <?php if (esc($product->product_unit) != esc($unit->unit_id)) : ?>
                                                    <option value="<?= $unit->unit_id ?>"><?= $unit->unit_name ?></option>
                                                <?php endif; ?>
                                            <?php endforeach; ?>
                                        </select>
                                    </div>
                                </div>
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Sell Type <span class="text-danger">*</span></label>
                                        <select class="custom-select" name="sale_type" required>
                                            <option value="" selected>Select one</option>
                                            <option value="1">Retail</option>
                                            <option value="2">Whole Sale</option>
                                        </select>
                                    </div>
                                </div>
                                <div class="col-lg-6">
                                    <div class="form-group">
                                        <label>Unit Value <span class="text-danger">*</span></label>
                                        <input type="number" name="unit_value" class="form-control" value="<?= esc($product->unit_value) ?>" required>
                                    </div>
                                </div>
                                <div class="col-lg-6">
                                    <div class="form-group">
                                        <label>Unit Price <span class="text-danger">*</span></label>
                                        <input type="number" name="price" class="form-control" value="<?= esc($product->product_price) ?>" required>
                                    </div>
                                </div>

                                <div class="col-lg-12">
                                    <div class="form-group">
                                        <label>Product Description <span class="text-danger">*</span></label>
                                        <textarea name="description" id="ckeditor" required><?= esc($product->product_desc) ?></textarea>
                                    </div>
                                </div>
                                <div class="col-lg-12">
                                    <div class="form-group">
                                        <label>Product Image <span class="text-success">(Optional)</span></label>
                                        <input type="file" name="product_image" class="form-control-file">
                                        <input type="text" name="old_image" class="form-control" value="<?= esc($product->product_image) ?>" hidden>
                                    </div>
                                    <img src="<?= base_url('public/uploads/products/products/' . esc($product->product_image)) ?>" alt="Product" class="img-fluid" width="150">
                                </div>
                            </div>
                            <br>
                            <button type="submit" class="btn btn-primary">Update Data</button>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>

<?= $this->endSection(); ?>